
/*
 * libgnet - game net library
 * Copyright (C) 1997 Indrek Mandre 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _GNET_H
#define _GNET_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <unistd.h>	/* For fd_set */
#include <sys/time.h>	/* for FD_??? */

#define NET_UNIX	1
#define NET_INTERNET	2

#define CON_SEQPACKET	1
#define CON_UDP		2	

#define GNET_VERSION	1

#define GN_TRUE		1
#define GN_FALSE	0

/*
 *  Server stuff comes from here...
 */

struct GN_client;

typedef struct GN_server {
	int	tcpsocket;
	int	tcpsocktype;
	int	tcpsockarg;

	int	isudp;

	fd_set	fdchk;

	struct GN_client	*clients;
} GN_server;

typedef struct GN_client {
	int		tcpsocket;
	int		udpsocket;
	GN_server	*server;
	char		*udpaddress;
	int		flags;

	char		fromadd[20];
	int		fromport;

	struct GN_client	*next;
	struct GN_client	*prev;
}GN_client;

extern char GN_error[256];
extern char *GN_unix_socket_name;

#define GN_is_client_data(cl) FD_ISSET(cl->tcpsocket,&cl->server->fdchk)
#define GN_is_client_udp_data(cl) FD_ISSET(cl->udpsocket,&cl->server->fdchk)

GN_server * GN_init_server ( int sock_type, int port, int sock_arg );
void GN_check_server ( GN_server *serv );
int GN_is_new_client ( GN_server *serv );
GN_client * GN_accept_client ( GN_server *server );
void GN_set_unix_socket_path ( char *path );
int GN_serv_send_data ( GN_client *cli, void *data, int len );
int GN_serv_send_udp_data ( GN_client *cli, void *data, int len );
int GN_serv_recv_data ( GN_client *cli, void *data, int len );
int GN_serv_recv_udp_data ( GN_client *cli, void *data, int len );
GN_client * GN_rem_client ( GN_client *client );
void GN_close_server ( GN_server *server );

/*
 *  Client stuff comes from here...
 */

typedef struct GN_conn {
	int	socket;
	int	udpsocket;
	int	udpport;
	char	*sendtoudp;
	fd_set	fdchk;
}GN_conn;

#define GN_is_data(cl) FD_ISSET(cl->socket,&cl->fdchk)
#define GN_is_udp_data(cl) FD_ISSET(cl->udpsocket,&cl->fdchk)

GN_conn * GN_connect ( int sock_type, char *host, int port, int sock_arg );
int GN_send_data ( GN_conn *to, void *data, int len );
int GN_send_udp_data ( GN_conn *to, void *data, int len );
int GN_recv_data ( GN_conn *to, void *data, int len );
int GN_recv_udp_data ( GN_conn *to, void *data, int len );
void GN_check_conn ( GN_conn *ppp );
void GN_close_conn ( GN_conn *ppp );

#ifdef __cplusplus
}
#endif

#endif
